==> touhoulittlemaid-fabric-1.20.1-0.1.0-forge1.3.7-sources.jar: com/github/tartaricacid/touhoulittlemaid/client/renderer/entity/EntityDanmakuRenderer.java <==
package com.github.tartaricacid.touhoulittlemaid.client.renderer.entity;

import com.github.tartaricacid.touhoulittlemaid.TouhouLittleMaid;
import com.github.tartaricacid.touhoulittlemaid.entity.projectile.DanmakuColor;
import com.github.tartaricacid.touhoulittlemaid.entity.projectile.DanmakuType;
import com.github.tartaricacid.touhoulittlemaid.entity.projectile.EntityDanmaku;
import net.minecraft.class_1921;
import net.minecraft.class_2960;
import net.minecraft.class_4587;
import net.minecraft.class_4588;
import net.minecraft.class_4597;
import net.minecraft.class_4608;
import net.minecraft.class_5617;
import net.minecraft.class_897;
import org.joml.Matrix3f;
import org.joml.Matrix4f;

public class EntityDanmakuRenderer extends class_897<EntityDanmaku> {
    private static final class_2960 DANMAKU_TEXTURE = new class_2960(TouhouLittleMaid.MOD_ID, "textures/entity/danmaku.png");
    private static final class_1921 RENDER_TYPE = class_1921.method_29379(DANMAKU_TEXTURE);

    public EntityDanmakuRenderer(class_5617.class_5618 renderManager) {
        super(renderManager);
    }

    private static void vertex(class_4588 bufferIn, Matrix4f pose, Matrix3f normal, double x, double y, double texU, double texV, int packedLight) {
        bufferIn.method_22918(pose, (float) x, (float) y, 0.0F).method_1336(255, 255, 255, 255).method_22913((float) texU, (float) texV).method_22922(class_4608.field_21444).method_22916(packedLight).method_23763(normal, 0.0F, 1.0F, 0.0F).method_1344();
    }

    @Override
    public void render(EntityDanmaku entity, float entityYaw, float partialTicks, class_4587 poseStack, class_4597 bufferIn, int packedLightIn) {
        // 获取相关数据
        DanmakuColor color = entity.getColor();
        DanmakuType type = entity.getDanmakuType();
        // 材质宽度
        int width = 416;
        // 材质长度
        int length = 128;

        // 依据类型颜色开始定位材质位置（材质块都是 32 * 32 大小）
        double startU = 32 * color.ordinal();
        double startV = 32 * type.ordinal();

        poseStack.method_22903();
        poseStack.method_22904(0, 0.1, 0);
        poseStack.method_22907(this.field_4676.method_24197());

        class_4588 buffer = bufferIn.getBuffer(RENDER_TYPE);
        class_4587.class_4665 poseStackLast = poseStack.method_23760();
        Matrix4f pose = poseStackLast.method_23761();
        Matrix3f normal = poseStackLast.method_23762();

        vertex(buffer, pose, normal, -type.getSize(), type.getSize(), (startU + 0) / width, (startV + 0) / length, packedLightIn);
        vertex(buffer, pose, normal, -type.getSize(), -type.getSize(), (startU + 0) / width, (startV + 32) / length, packedLightIn);
        vertex(buffer, pose, normal, type.getSize(), -type.getSize(), (startU + 32) / width, (startV + 32) / length, packedLightIn);
        vertex(buffer, pose, normal, type.getSize(), type.getSize(), (startU + 32) / width, (startV + 0) / length, packedLightIn);
        poseStack.method_22909();
    }

    @Override
    public class_2960 getTextureLocation(EntityDanmaku entity) {
        return DANMAKU_TEXTURE;
    }
}

==> touhoulittlemaid-fabric-1.20.1-0.1.4-forge1.3.8-sources.jar: com/github/tartaricacid/touhoulittlemaid/client/renderer/entity/EntityDanmakuRenderer.java <==
package com.github.tartaricacid.touhoulittlemaid.client.renderer.entity;

import com.github.tartaricacid.touhoulittlemaid.TouhouLittleMaid;
import com.github.tartaricacid.touhoulittlemaid.entity.projectile.DanmakuColor;
import com.github.tartaricacid.touhoulittlemaid.entity.projectile.DanmakuType;
import com.github.tartaricacid.touhoulittlemaid.entity.projectile.EntityDanmaku;
import net.minecraft.class_1921;
import net.minecraft.class_2960;
import net.minecraft.class_4587;
import net.minecraft.class_4588;
import net.minecraft.class_4597;
import net.minecraft.class_4608;
import net.minecraft.class_5617;
import net.minecraft.class_7833;
import net.minecraft.class_897;
import org.joml.Matrix3f;
import org.joml.Matrix4f;

public class EntityDanmakuRenderer extends class_897<EntityDanmaku> {
    private static final class_2960 DANMAKU_TEXTURE = new class_2960(TouhouLittleMaid.MOD_ID, "textures/entity/danmaku.png");
    private static final class_1921 RENDER_TYPE = class_1921.method_29379(DANMAKU_TEXTURE);

    public EntityDanmakuRenderer(class_5617.class_5618 renderManager) {
        super(renderManager);
    }

    private static void vertex(class_4588 bufferIn, Matrix4f pose, Matrix3f normal, double x, double y, double texU, double texV, int packedLight) {
        bufferIn.method_22918(pose, (float) x, (float) y, 0.0F).method_1336(255, 255, 255, 255).method_22913((float) texU, (float) texV).method_22922(class_4608.field_21444).method_22916(packedLight).method_23763(normal, 0.0F, 1.0F, 0.0F).method_1344();
    }

    @Override
    public void render(EntityDanmaku entity, float entityYaw, float partialTicks, class_4587 poseStack, class_4597 bufferIn, int packedLightIn) {
        // 获取相关数据
        DanmakuColor color = entity.getColor();
        DanmakuType type = entity.getDanmakuType();
        // 材质宽度
        int width = 416;
        // 材质长度
        int length = 128;

        // 依据类型颜色开始定位材质位置（材质块都是 32 * 32 大小）
        double startU = 32 * color.ordinal();
        double startV = 32 * type.ordinal();

        poseStack.method_22903();
        poseStack.method_22904(0, 0.1, 0);
        poseStack.method_22907(this.field_4676.method_24197());
        poseStack.method_22907(class_7833.field_40716.rotationDegrees(180.0F));

        class_4588 buffer = bufferIn.getBuffer(RENDER_TYPE);
        class_4587.class_4665 poseStackLast = poseStack.method_23760();
        Matrix4f pose = poseStackLast.method_23761();
        Matrix3f normal = poseStackLast.method_23762();

        vertex(buffer, pose, normal, -type.getSize(), type.getSize(), (startU + 0) / width, (startV + 0) / length, packedLightIn);
        vertex(buffer, pose, normal, -type.getSize(), -type.getSize(), (startU + 0) / width, (startV + 32) / length, packedLightIn);
        vertex(buffer, pose, normal, type.getSize(), -type.getSize(), (startU + 32) / width, (startV + 32) / length, packedLightIn);
        vertex(buffer, pose, normal, type.getSize(), type.getSize(), (startU + 32) / width, (startV + 0) / length, packedLightIn);
        poseStack.method_22909();
    }

    @Override
    public class_2960 getTextureLocation(EntityDanmaku entity) {
        return DANMAKU_TEXTURE;
    }
}
